/*
 * kill.c
 *
 * Copyright (C) 2006  Insigme Co., Ltd
 *
 * This software has been developed while working on the Linux Unified Kernel
 * project (http://www.longene.org) in the Insigma Research Institute,
 * which is a subdivision of Insigma Co., Ltd (http://www.insigma.com.cn).
 *
 * The project is sponsored by Insigma Co., Ltd.
 *
 * The authors can be reached at linux@insigma.com.cn.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of  the GNU General  Public License as published by the
 * Free Software Foundation; either version 2 of the  License, or (at your
 * option) any later version.
 *
 * Revision History:
 *   Jan 2006 - Created.
 */

/*
 * kill.c:
 * Refered to ReactOS code
 */
#include "win32_process.h"
#include <linux/mm.h>
#include <linux/syscalls.h>
#include <asm/uaccess.h>
#include "ke.h"
#include "win32.h"
#include "thread.h"
#include "object.h"
#include "process.h"
#include "section.h"
#include "handle.h"
#include "objwait.h"
#include "virtual.h"
#include "handle.h"
#include "event.h"
#include "w32syscall.h"
#include "attach.h"
#include "area.h"
#include "wineserver/info.h"

#ifdef CONFIG_UNIFIED_KERNEL

extern VOID STDCALL
delete_handle_callback(struct handle_table *HandleTable,
		PVOID Object,
		ULONG GrantedAccess,
		PVOID Context);

VOID STDCALL
delete_teb(PTEB Teb)
{
	unsigned long len = PAGE_SIZE;

	ktrace("delete_teb %p\n", Teb);
	NtFreeVirtualMemory((HANDLE)-1, (PVOID *)&Teb, (PULONG)&len, MEM_RELEASE);
}

void __exit_process(struct eprocess * process)
{
	unsigned long old_state;

	/* close all handles associated with our process, this needs to be done 
	 * when the last thread still runs */
	__destroy_handle_table(process->object_table, delete_handle_callback, process);

	/* remove all reserved area and mapped area */
	remove_all_win32_area(&process->ep_reserved_head);
	remove_all_win32_area(&process->ep_mapped_head);

	/* FIXME: spin_lock_irq(&t); */
	local_irq_disable();

	if (process->win32process)
		kfree(process->win32process);

    if (process->ep_handle_info_table) {
        free_handle_info_table(process->ep_handle_info_table);
        kfree(process->ep_handle_info_table);
        process->ep_handle_info_table = NULL;
    }

	old_state = process->pcb.header.signal_state;
	process->pcb.header.signal_state = true;
	if ((!old_state) && !list_empty(&process->pcb.header.wait_list_head)) {
		/* Satisfy waits */
		wait_test((struct dispatcher_header *)&process->pcb,IO_NO_INCREMENT);
	}

	local_irq_enable();
	/* FIXME: spin_unlock_irq(&t); */
}
EXPORT_SYMBOL(__exit_process);


static inline void terminate_thread_by_pointer(struct ethread *thread, NTSTATUS exit_code)
{
	thread->exit_status = exit_code;
	send_sig_info(SIGKILL, SEND_SIG_FORCED, thread->et_task);
}


void exit_process_threads(struct eprocess *process, NTSTATUS ExitStatus)
{
	struct list_head *cur_entry;
	struct ethread *thread;

	/* process is locked by caller */
	cur_entry = process->thread_list_head.next;
	while (cur_entry != &process->thread_list_head) {
		thread = list_entry(cur_entry, struct ethread, thread_list_entry);
		cur_entry = cur_entry->next;
		if (thread != get_current_ethread())
			terminate_thread_by_pointer(thread, ExitStatus);
	}
}
EXPORT_SYMBOL(exit_process_threads);


NTSTATUS 
SERVICECALL
NtTerminateThread(IN HANDLE   ThreadHandle,
		IN NTSTATUS ExitStatus)
{
	struct ethread * thread;
	NTSTATUS status;

	ktrace("NtTerminateThread handle %p\n", ThreadHandle);

	status = ref_object_by_handle(ThreadHandle,
			THREAD_TERMINATE,
			thread_object_type,
			get_pre_mode(),
			(PVOID *)&thread,
			NULL);

	if (!NT_SUCCESS(status))
		return status;

	if (!thread->et_task){
		deref_object(thread);
	       	return STATUS_THREAD_IS_TERMINATING; 
	}

	/* FIXME Make sure this is not a system thread */

	if (thread != get_current_ethread()) {
		terminate_thread_by_pointer(thread, ExitStatus); 
		deref_object(thread);
	} else {
		deref_object(thread);
		thread->exit_status = ExitStatus;
		do_exit((ExitStatus & 0xff) << 8);
	}

	return STATUS_SUCCESS;		
}
EXPORT_SYMBOL(NtTerminateThread);

#endif

